@@ -17,6 +17,16 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+/*
+ * @file   ime.c
+ * @author Yuking yuking_net@sohu.com
+ * @date   2008-1-16
+ * 
+ * @brief  뷨ͨùܴ
+ * 
+ * 
+ */
+
 #include <ctype.h>
 #include <time.h>
 
@@ -39,6 +49,8 @@
 #include "ui.h"
 #include "vk.h"
 #include "QuickPhrase.h"
+#include "AutoEng.h"
+#include "extra.h"
 
 IM             *im = NULL;
 INT8            iIMCount = 0;
@@ -67,8 +79,12 @@ Bool            bChnPunc = True;	//Ӣıл
 Bool            bUseGBK = False;	//Ƿ֧GBK
 Bool            bIsDoInputOnly = False;	//Ƿֻ뷨
 Bool            bLastIsNumber = False;	//һǲǰ
-INT8            iInCap = 0;	//ǲǴڴдӢ״̬,0--1--´д2--·ֺż
+char		cLastIsAutoConvert = 0;	//һǲǷֺԶתķţ'.'/','0ʾķ
+INT8            iInCap = 0;		//ǲǴڴдӢ״̬,0--1--´д2--·ֺż
+
+/*
 Bool            bAutoHideInputWindow = False;	//ǷԶ
+*/
 Bool            bEngPuncAfterNumber = True;	//ֺǷ(ֻ'.'/','Ч)
 Bool            bPhraseTips = True;
 INT8            lastIsSingleHZ = 0;
@@ -101,6 +117,7 @@ HOTKEYS         hkNextPage[HOT_KEY_COUNT] = { '.', 0 };	//һҳ
 HOTKEYS         hkPrevPage[HOT_KEY_COUNT] = { ',', 0 };	//һҳ
 HOTKEYS         hkTrack[HOT_KEY_COUNT] = { CTRL_K, 0 };
 HOTKEYS         hkGBT[HOT_KEY_COUNT] = { CTRL_ALT_F, 0 };
+HOTKEYS         hkHideMainWindow[HOT_KEY_COUNT] = { CTRL_ALT_H, 0 };
 
 Bool            bUseLegend = False;
 Bool            bIsInLegend = False;
@@ -110,7 +127,10 @@ Bool            bUsePinyin = True;
 Bool            bUseSP = True;
 Bool            bUseQW = True;
 Bool            bUseTable = True;
-Bool            bLocked = True;
+Bool            bLocked = False;
+
+// dgod extern im
+char		strExternIM[PATH_MAX];
 
 /* µLumaQQѾҪ֧
 Bool            bLumaQQ = False;
@@ -132,6 +152,9 @@ char            strNameOfQuwei[41] = "λ";
 INT8		iKeyPressed = 0;
 Bool		bRelease = True;
 */
+int             keyIgnored[] = { 50, 62, 37, 109, 115, 116, 117, 0 };
+
+Bool		bCursorAuto=False;
 
 extern XIMS     ims;
 extern IC      *CurrentIC;
@@ -179,6 +202,9 @@ extern int      iInputWindowY;
 extern int      iTempInputWindowX;
 extern int      iTempInputWindowY;
 
+extern Bool     bShowInputWindowTriggering;
+extern Bool	bMainWindow_Hiden;
+
 #ifdef _USE_XFT
 extern XftFont *xftMainWindowFont;
 #else
@@ -217,8 +243,11 @@ void ResetInput (void)
 
 void CloseIM (IMForwardEventStruct * call_data)
 {
-    XUnmapWindow (dpy, inputWindow);
-    XUnmapWindow (dpy, VKWindow);
+    if (IsWindowVisible (inputWindow))
+	XUnmapWindow (dpy, inputWindow);
+    if (IsWindowVisible (VKWindow))
+	XUnmapWindow (dpy, VKWindow);
+
     IMPreeditEnd (ims, (XPointer) call_data);
     SetConnectID (call_data->connect_id, IS_CLOSED);
     bVK = False;
@@ -244,8 +273,11 @@ void ChangeIMState (CARD16 _connect_id)
 	SetConnectID (_connect_id, IS_ENG);
 	ResetInput ();
 	ResetInputWindow ();
-	XUnmapWindow (dpy, inputWindow);
-	XUnmapWindow (dpy, VKWindow);
+
+	if (IsWindowVisible (inputWindow))
+	    XUnmapWindow (dpy, inputWindow);
+	if (IsWindowVisible (VKWindow))
+	    XUnmapWindow (dpy, VKWindow);
     }
 
     if (hideMainWindow != HM_HIDE)
@@ -278,6 +310,19 @@ void ConvertPunc (void)
     strcpy (strStringGet, strTemp);
 }
 
+Bool IsKeyIgnored (int iKeyCode)
+{
+    int             i;
+
+    i = 0;
+    while (keyIgnored[i]) {
+	if (iKeyCode == keyIgnored[i])
+	    return True;
+	i++;
+    }
+    return False;
+}
+
 //FILE           *fd;
 void ProcessKey (IMForwardEventStruct * call_data)
 {
@@ -298,23 +343,28 @@ void ProcessKey (IMForwardEventStruct * call_data)
     iKeyState = kev->state - (kev->state & KEY_NUMLOCK) - (kev->state & KEY_CAPSLOCK) - (kev->state & KEY_SCROLLLOCK);
     iKey = GetKey (keysym, iKeyState, keyCount);
 
-    if (!iKey)
+    if (!iKey) {
+	IMForwardEvent (ims, (XPointer) call_data);
 	return;
+    }
 
     /*
-     * xineкѡԶѡе
-     * xineÿӲһSHIFTͷ¼
+     * ԸΪ˽xine-uiкѡԶѡе
+     * xine-uiÿӲһSHIFTͷ¼
+     * δµxine-uiѾ
      */
-    if (kev->same_screen && (kev->keycode == switchKey || kev->keycode == i2ndSelectKey || kev->keycode == i3rdSelectKey))
+     /* if (kev->same_screen && (kev->keycode == switchKey || kev->keycode == i2ndSelectKey || kev->keycode == i3rdSelectKey)) {
+	IMForwardEvent (ims, (XPointer) call_data);
 	return;
+     } */
 
     retVal = IRV_TO_PROCESS;
-    
+
 #ifdef _DEBUG
-    printf("iKeyState=%d  KEYCODE=%d  iKey=%d\n", iKeyState, kev->keycode, iKey);
+    printf ("KeyRelease=%d  iKeyState=%d  KEYCODE=%d  iKey=%d\n", (call_data->event.type == KeyRelease), iKeyState, kev->keycode, iKey);
 #endif
-    
-    if (call_data->event.type == KeyRelease) {
+
+    if ((call_data->event.type == KeyRelease) && (ConnectIDGetState (call_data->connect_id) != IS_CLOSED)) {
 	if ((kev->time - lastKeyPressedTime) < 500 && (!bIsDoInputOnly)) {
 	    if (!bLocked && iKeyState == KEY_CTRL_SHIFT_COMP && (iKey == 225 || iKey == 227)) {
 		if (ConnectIDGetState (call_data->connect_id) == IS_CHN)
@@ -329,11 +379,17 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		    CloseIM (call_data);
 	    }
 	    else if (kev->keycode == switchKey && keyReleased == KR_CTRL && !bDoubleSwitchKey) {
-		ChangeIMState (call_data->connect_id);
-		retVal = IRV_DO_NOTHING;
+	    	if (iCodeInputCount) {
+	    	    strcpy (strStringGet, strCodeInput);
+		    retVal = IRV_ENG;
+		}
+		else
+		    retVal = IRV_TO_PROCESS;
+		    
+		keyReleased = KR_OTHER;
+		ChangeIMState (call_data->connect_id);		
 	    }
-	    else if ((kev->keycode == i2ndSelectKey && keyReleased == KR_2ND_SELECTKEY) ||
-		(iKey == i2ndSelectKey && keyReleased == KR_2ND_SELECTKEY_OTHER)) {
+	    else if ((kev->keycode == i2ndSelectKey && keyReleased == KR_2ND_SELECTKEY) || (iKey == (i2ndSelectKey ^ 0xFF) && keyReleased == KR_2ND_SELECTKEY_OTHER)) {
 		if (!bIsInLegend) {
 		    pstr = im[iIMIndex].GetCandWord (1);
 		    if (pstr) {
@@ -353,11 +409,10 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		    uMessageDown = 0;
 		    retVal = IRV_GET_CANDWORDS;
 		}
-		
-		keyReleased=KR_OTHER;
+
+		keyReleased = KR_OTHER;
 	    }
-	    else if ((kev->keycode == i3rdSelectKey && keyReleased == KR_3RD_SELECTKEY) ||
-		(iKey == i3rdSelectKey && keyReleased == KR_3RD_SELECTKEY_OTHER)) {
+	    else if ((iKey == (i3rdSelectKey ^ 0xFF) && keyReleased == KR_3RD_SELECTKEY_OTHER) || (kev->keycode == i3rdSelectKey && keyReleased == KR_3RD_SELECTKEY)) {
 		if (!bIsInLegend) {
 		    pstr = im[iIMIndex].GetCandWord (2);
 		    if (pstr) {
@@ -375,8 +430,30 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		    uMessageDown = 0;
 		    retVal = IRV_GET_CANDWORDS;
 		}
-		
-		keyReleased=KR_OTHER;
+
+		keyReleased = KR_OTHER;
+	    }
+
+	    else if (kev->keycode == i3rdSelectKey && keyReleased == KR_3RD_SELECTKEY) {
+		if (!bIsInLegend) {
+		    pstr = im[iIMIndex].GetCandWord (2);
+		    if (pstr) {
+			strcpy (strStringGet, pstr);
+			if (bIsInLegend)
+			    retVal = IRV_GET_LEGEND;
+			else
+			    retVal = IRV_GET_CANDWORDS;
+		    }
+		    else if (iCandWordCount)
+			retVal = IRV_DISPLAY_CANDWORDS;
+		}
+		else {
+		    strcpy (strStringGet, "");
+		    uMessageDown = 0;
+		    retVal = IRV_GET_CANDWORDS;
+		}
+
+		keyReleased = KR_OTHER;
 	    }
 	}
     }
@@ -386,29 +463,33 @@ void ProcessKey (IMForwardEventStruct * call_data)
 	    if (kev->keycode != switchKey)
 		keyReleased = KR_OTHER;
 	    else {
-		if ((keyReleased == KR_CTRL) && (kev->time - lastKeyPressedTime < iTimeInterval) && bDoubleSwitchKey)
+		if ((keyReleased == KR_CTRL) && (kev->time - lastKeyPressedTime < iTimeInterval) && bDoubleSwitchKey) {
+	    	    SendHZtoClient(call_data, strCodeInput);
 		    ChangeIMState (call_data->connect_id);
+	        }
 	    }
 
 	    lastKeyPressedTime = kev->time;
 	    if (kev->keycode == switchKey) {
 		keyReleased = KR_CTRL;
-		//retVal = (bDoubleSwitchKey)? IRV_CLEAN:IRV_DO_NOTHING;
 		retVal = IRV_DO_NOTHING;
 	    }
 	    else if (IsHotKey (iKey, hkTrigger)) {
 		if (ConnectIDGetState (call_data->connect_id) == IS_ENG) {
 		    SetConnectID (call_data->connect_id, IS_CHN);
-		    DisplayInputWindow ();
+
+		    if (bShowInputWindowTriggering && !bCorner) {
+			DrawInputWindow ();
+			DisplayInputWindow ();
+		    }
 
 		    if (ConnectIDGetTrackCursor (call_data->connect_id))
 			XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 		    else
 			XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
 
-		    if (hideMainWindow != HM_HIDE)
-			DisplayMainWindow ();
-
+		    EnterChineseMode (False);
+		    DrawMainWindow ();
 		}
 		else
 		    CloseIM (call_data);
@@ -428,14 +509,14 @@ void ProcessKey (IMForwardEventStruct * call_data)
 				keyReleased = KR_2ND_SELECTKEY;
 			    else if (kev->keycode == i3rdSelectKey)
 				keyReleased = KR_3RD_SELECTKEY;
-			    else if (iKey == i2ndSelectKey) {
-				if (iCandWordCount>=2) {
+			    else if (iKey == (i2ndSelectKey ^ 0xFF)) {
+				if (iCandWordCount >= 2) {
 				    keyReleased = KR_2ND_SELECTKEY_OTHER;
 				    return;
 				}
 			    }
-			    else if (iKey == i3rdSelectKey) {
-				if (iCandWordCount>=2) {
+			    else if (iKey == (i3rdSelectKey ^ 0xFF)) {
+				if (iCandWordCount >= 2) {
 				    keyReleased = KR_3RD_SELECTKEY_OTHER;
 				    return;
 				}
@@ -460,9 +541,29 @@ void ProcessKey (IMForwardEventStruct * call_data)
 			    }
 			    else {
 				if (!iInCap) {
+				    char            strTemp[MAX_USER_INPUT];
+
 				    retVal = im[iIMIndex].DoInput (iKey);
-				    if (!IsIM (strNameOfPinyin) && !IsIM (strNameOfShuangpin))
+				    if (!bCursorAuto && !IsIM (strNameOfPinyin) && !IsIM (strNameOfShuangpin))
 					iCursorPos = iCodeInputCount;
+
+				    //ΪʵԶӢת
+				    strcpy (strTemp, strCodeInput);
+				    if (retVal == IRV_TO_PROCESS) {
+					strTemp[strlen (strTemp) + 1] = '\0';
+					strTemp[strlen (strTemp)] = iKey;
+				    }
+
+				    if (SwitchToEng (strTemp)) {
+					iInCap = 3;
+					if (retVal != IRV_TO_PROCESS) {
+					    iCodeInputCount--;
+					    retVal = IRV_TO_PROCESS;
+					}
+				    }
+				    
+				    if (iKey!= (XK_BackSpace & 0x00FF))
+					cLastIsAutoConvert = 0;
 				}
 				else if (iInCap == 2 && semicolonToDo == K_SEMICOLON_QUICKPHRASE)
 				    retVal = QuickPhraseDoInput (iKey);
@@ -474,10 +575,10 @@ void ProcessKey (IMForwardEventStruct * call_data)
 					    pstr = im[iIMIndex].GetCandWord (0);
 					    iCandWordCount = 0;
 					    if (pstr) {
+						SendHZtoClient (call_data, pstr);
 						strcpy (strStringGet, pstr);
 						//ͳ
 						iHZInputed += (int) (strlen (strStringGet) / 2);
-						SendHZtoClient (call_data, strStringGet);
 						iCodeInputCount = 0;
 					    }
 					}
@@ -522,28 +623,38 @@ void ProcessKey (IMForwardEventStruct * call_data)
 							    retVal = IRV_DISPLAY_MESSAGE;
 						    }
 						}
-						else if ((iKey == (XK_BackSpace & 0x00FF) || iKey == CTRL_H) && (iCodeInputCount || (!iCodeInputCount && iInCap))) {
+						else if (iKey == (XK_BackSpace & 0x00FF) || iKey == CTRL_H) {
 						    if (iCodeInputCount)
 							iCodeInputCount--;
 						    strCodeInput[iCodeInputCount] = '\0';
 						    iCursorPos = iCodeInputCount;
-						    retVal = IRV_DISPLAY_MESSAGE;
 						    if (!iCodeInputCount)
 							retVal = IRV_CLEAN;
 						    else if (semicolonToDo == K_SEMICOLON_QUICKPHRASE && iInCap == 2)
 							retVal = QuickPhraseGetCandWords (SM_FIRST);
+						    else
+							retVal = IRV_DISPLAY_MESSAGE;
 						}
 
 						uMessageUp = 1;
 						if (iInCap == 2) {
-						    strcpy (messageUp[0].strMsg, strCodeInput);
-						    if (semicolonToDo == K_SEMICOLON_ENG)
-							strcat (messageUp[0].strMsg, "Ӣ");
-						    else {
-							if (iCodeInputCount)
-							    strcat (messageUp[0].strMsg, "  ");
-							strcat (messageUp[0].strMsg, "Զ");
+						    if (semicolonToDo == K_SEMICOLON_ENG) {
+							strcpy (messageUp[0].strMsg, "Ӣ ");
+							iCursorPos += 9;
 						    }
+						    else {
+							strcpy (messageUp[0].strMsg, "Զ ");
+							iCursorPos += 11;
+						    }
+
+						    if (iCodeInputCount) {
+							uMessageUp = 2;
+							//strcat (messageUp[0].strMsg, "  ");
+							strcpy (messageUp[1].strMsg, strCodeInput);
+							messageUp[1].type = MSG_INPUT;
+							iCursorPos += iCodeInputCount;
+						    }
+
 						    if (retVal != IRV_DISPLAY_CANDWORDS) {
 							if (iCodeInputCount)
 							    strcpy (messageDown[0].strMsg, " Enter Ӣ");
@@ -552,21 +663,23 @@ void ProcessKey (IMForwardEventStruct * call_data)
 							uMessageDown = 1;
 							messageDown[0].type = MSG_TIPS;
 						    }
+						    messageUp[0].type = MSG_TIPS;
 						}
-						else {
+						else {						    
 						    uMessageDown = 1;
 						    messageDown[0].type = MSG_TIPS;
 						    strcpy (messageUp[0].strMsg, strCodeInput);
 						    strcpy (messageDown[0].strMsg, " Enter Ӣ");
+						    messageUp[0].type = MSG_INPUT;
 						}
-						messageUp[0].type = MSG_INPUT;
 					    }
 					}
-					else if ((bLastIsNumber && bEngPuncAfterNumber) && (iKey == '.' || iKey == ',') && !iCandWordCount) {
-					    retVal = IRV_TO_PROCESS;
+					else if ((bLastIsNumber && bEngPuncAfterNumber) && (iKey == '.' || iKey == ',' || iKey == ':') && !iCandWordCount) {
+					    cLastIsAutoConvert = iKey;
 					    bLastIsNumber = False;
+					    retVal = IRV_TO_PROCESS;					    
 					}
-					else {
+					else {				    
 					    if (bChnPunc) {
 						char           *pPunc;
 
@@ -583,6 +696,16 @@ void ProcessKey (IMForwardEventStruct * call_data)
 
 						    retVal = IRV_PUNC;
 						}
+						else if ((iKey == (XK_BackSpace & 0x00FF) || iKey == CTRL_H) && cLastIsAutoConvert ) {
+						    char *pPunc;
+						    
+						    IMForwardEvent (ims, (XPointer) call_data);
+						    pPunc = GetPunc(cLastIsAutoConvert);
+						    if ( pPunc )
+						    	SendHZtoClient(call_data, pPunc);
+							
+						    retVal = IRV_DO_NOTHING;
+					        }
 						else if (isprint (iKey) && iKey < 128) {
 						    if (iKey >= '0' && iKey <= '9')
 							bLastIsNumber = True;
@@ -605,6 +728,7 @@ void ProcessKey (IMForwardEventStruct * call_data)
 						    }
 						}
 					    }
+					    cLastIsAutoConvert = 0;
 					}
 				    }
 				}
@@ -618,21 +742,31 @@ void ProcessKey (IMForwardEventStruct * call_data)
 				    }
 				    else if (iKey == CTRL_5) {
 					LoadConfig (False);
+
+					InitGC (inputWindow);
+					InitMainWindowColor ();
+					InitInputWindowColor ();
+					InitVKWindowColor ();
+
 					SetIM ();
 
 					/*if (bLumaQQ)
 					   ConnectIDResetReset (); */
 
 					CreateFont ();
-					SwitchIM (iIMIndex);
 					CalculateInputWindowHeight ();
 
 					FreeQuickPhrase ();
 					LoadQuickPhrase ();
 
+					FreeAutoEng ();
+					LoadAutoEng ();
+
 					FreePunc ();
 					LoadPuncDict ();
 
+					DrawMainWindow();
+
 					retVal = IRV_DO_NOTHING;
 				    }
 				    else if (iKey == ENTER) {
@@ -686,6 +820,18 @@ void ProcessKey (IMForwardEventStruct * call_data)
 			retVal = ChangeTrack ();
 		    else if (IsHotKey (iKey, hkGBT))
 			retVal = ChangeGBKT ();
+		    else if (IsHotKey (iKey, hkHideMainWindow)) {
+			if (IsWindowVisible(mainWindow)) {
+			    bMainWindow_Hiden = True;
+			    XUnmapWindow(dpy,mainWindow);
+		    	}
+			else {
+			    bMainWindow_Hiden = False;
+			    DisplayMainWindow();
+			    DrawMainWindow();
+			}
+			retVal = IRV_DO_NOTHING;			
+		    }
 		}
 	    }
 	    else
@@ -699,23 +845,15 @@ void ProcessKey (IMForwardEventStruct * call_data)
     case IRV_TO_PROCESS:
     case IRV_DONOT_PROCESS:
     case IRV_DONOT_PROCESS_CLEAN:
-	if (call_data->event.type == KeyRelease) {
-	    // if (!bLumaQQ && (!keyCount || (!iKeyState && (iKey == ESC || iKey == ENTER))))
-	    if ((!keyCount && kev->keycode != L_CTRL && kev->keycode != R_CTRL) || (!iKeyState && (iKey == ESC || iKey == ENTER)))
-		IMForwardEvent (ims, (XPointer) call_data);
-	}
-	else
-	    IMForwardEvent (ims, (XPointer) call_data);
+	IMForwardEvent (ims, (XPointer) call_data);
 
 	if (retVal != IRV_DONOT_PROCESS_CLEAN)
 	    return;
     case IRV_CLEAN:
 	ResetInput ();
 	ResetInputWindow ();
-	if (bAutoHideInputWindow)
+	if (IsWindowVisible (inputWindow))
 	    XUnmapWindow (dpy, inputWindow);
-	else
-	    DrawInputWindow ();
 
 	return;
     case IRV_DISPLAY_CANDWORDS:
@@ -733,8 +871,9 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		bShowNext = True;
 	}
 
-	DisplayInputWindow ();
 	DrawInputWindow ();
+	DisplayInputWindow ();
+
 	break;
     case IRV_DISPLAY_LAST:
 	bShowNext = bShowPrev = False;
@@ -745,13 +884,14 @@ void ProcessKey (IMForwardEventStruct * call_data)
 	uMessageDown = 1;
 	strcpy (messageDown[0].strMsg, strStringGet);
 	messageDown[0].type = MSG_TIPS;
+	DrawInputWindow ();
 	DisplayInputWindow ();
 	break;
     case IRV_DISPLAY_MESSAGE:
 	bShowNext = False;
 	bShowPrev = False;
-	DisplayInputWindow ();
 	DrawInputWindow ();
+	DisplayInputWindow ();
 	break;
     case IRV_GET_LEGEND:
 	SendHZtoClient (call_data, strStringGet);
@@ -765,41 +905,56 @@ void ProcessKey (IMForwardEventStruct * call_data)
 	    bLastIsNumber = False;
 	    iCodeInputCount = 0;
 	    DrawInputWindow ();
+	    DisplayInputWindow ();
 	}
 	else {
 	    ResetInput ();
-	    if (bAutoHideInputWindow)
+	    //if (bAutoHideInputWindow)
+	    if (IsWindowVisible (inputWindow))
 		XUnmapWindow (dpy, inputWindow);
-	    else
-		DrawInputWindow ();
+	    /*else
+	       DrawInputWindow ();
+	     */
 	}
 
 	break;
     case IRV_GET_CANDWORDS:
+	SendHZtoClient (call_data, strStringGet);
+	bLastIsNumber = False;
 	if (bPhraseTips && im[iIMIndex].PhraseTips && !bVK)
 	    DoPhraseTips ();
+	iHZInputed += (int) (strlen (strStringGet) / 2);	//ͳ
+	ResetInput ();
+
+	if (bVK || (!uMessageDown && (!bPhraseTips || (bPhraseTips && !lastIsSingleHZ)))) {
+	    if (IsWindowVisible (inputWindow))
+		XUnmapWindow (dpy, inputWindow);
+	}
+	else
+	    DrawInputWindow ();
+
+	lastIsSingleHZ = 0;
+
+	break;
     case IRV_ENG:
 	//ıģʽӦýеıתΪȫ
-	if (retVal != IRV_GET_CANDWORDS && bChnPunc && bConvertPunc)
+	if (bChnPunc && bConvertPunc)
 	    ConvertPunc ();
     case IRV_PUNC:
 	iHZInputed += (int) (strlen (strStringGet) / 2);	//ͳ
 	ResetInput ();
-	if (bVK || (!(uMessageDown && retVal == IRV_GET_CANDWORDS)
-		    && bAutoHideInputWindow && (retVal == IRV_PUNC || (!bPhraseTips || (bPhraseTips && !lastIsSingleHZ)))))
+	if (!uMessageDown && IsWindowVisible (inputWindow))
 	    XUnmapWindow (dpy, inputWindow);
-	else if (ConnectIDGetState (call_data->connect_id) == IS_CHN)
-	    DrawInputWindow ();
     case IRV_GET_CANDWORDS_NEXT:
+	SendHZtoClient (call_data, strStringGet);
+	bLastIsNumber = False;
+	lastIsSingleHZ = 0;
+
 	if (retVal == IRV_GET_CANDWORDS_NEXT || lastIsSingleHZ == -1) {
 	    iHZInputed += (int) (strlen (strStringGet) / 2);	//ͳ
 	    DrawInputWindow ();
 	}
 
-	SendHZtoClient (call_data, strStringGet);
-	bLastIsNumber = False;
-	lastIsSingleHZ = 0;
-
 	break;
     default:
 	;
@@ -850,7 +1005,8 @@ INPUT_RETURN_VALUE ChangeGBK (void)
     ResetInputWindow ();
 
     DrawMainWindow ();
-    XUnmapWindow (dpy, inputWindow);
+    if (IsWindowVisible (inputWindow))
+	XUnmapWindow (dpy, inputWindow);
 
     SaveProfile ();
 
@@ -864,7 +1020,8 @@ INPUT_RETURN_VALUE ChangeGBKT (void)
     ResetInputWindow ();
 
     DrawMainWindow ();
-    XUnmapWindow (dpy, inputWindow);
+    if (IsWindowVisible (inputWindow))
+	XUnmapWindow (dpy, inputWindow);
 
     SaveProfile ();
 
@@ -878,7 +1035,8 @@ INPUT_RETURN_VALUE ChangeLegend (void)
     ResetInputWindow ();
 
     DrawMainWindow ();
-    XUnmapWindow (dpy, inputWindow);
+    if (IsWindowVisible (inputWindow))
+	XUnmapWindow (dpy, inputWindow);
 
     SaveProfile ();
 
@@ -943,7 +1101,8 @@ void SwitchIM (INT8 index)
     }
 
     ResetInput ();
-    XUnmapWindow (dpy, inputWindow);
+    if (IsWindowVisible (inputWindow))
+	XUnmapWindow (dpy, inputWindow);
 
     SaveProfile ();
 }
@@ -1016,11 +1175,15 @@ void SetIM (void)
     if (bUseQW)
 	iIMCount++;
 
+    iIMCount += EIM_MAX;
+    if (!iIMCount)
+	iIMCount = 1;
+
     im = (IM *) malloc (sizeof (IM) * iIMCount);
     iIMCount = 0;
-
+    
     /* 뷨 */
-    if (bUsePinyin || (!bUseSP && (!bUseTable || !iTableCount)))	//Ӧһ뷨
+    if (bUsePinyin || ((!bUseSP && (!bUseTable || !iTableCount)) && !iIMCount))	//Ӧһ뷨
 	RegisterNewIM (strNameOfPinyin, ResetPYStatus, DoPYInput, PYGetCandWords, PYGetCandWord, PYGetLegendCandWord, NULL, PYInit, NULL);
     if (bUseSP)
 	RegisterNewIM (strNameOfShuangpin, ResetPYStatus, DoPYInput, PYGetCandWords, PYGetCandWord, PYGetLegendCandWord, NULL, SPInit, NULL);
@@ -1033,5 +1196,8 @@ void SetIM (void)
 	}
     }
 
+    if (strExternIM[0] && strExternIM[1])
+	    LoadExtraIM(strExternIM);
+	    
     SwitchIM (iIMIndex);
 }
