@@ -2,7 +2,7 @@
  *   С뷨(Free Chinese Input Toys for X, FCITX)                   *
  *   Yuking(yuking_net@sohu.com)д                                           *
  *   Э: GPL                                                              *
- *   FCITX( A Chinese XIM Input Method) by Yuking (yuking_net@sohu.com)    *   
+ *   FCITX( A Chinese XIM Input Method) by Yuking (yuking_net@sohu.com)    *
  *                                                                         *
  *   Copyright (C) 2002~2005 by Yuking                                     *
  *   yuking_net@sohu.com                                                   *
@@ -22,6 +22,16 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+/**
+ * @file   main.c
+ * @author Yuking yuking_net@sohu.com
+ * @date   2008-1-16
+ *
+ * @brief  
+ *
+ *
+ */
+
 #include <langinfo.h>
 
 #ifdef HAVE_CONFIG_H
@@ -44,17 +54,14 @@
 #include "sp.h"
 #include "about.h"
 #include "QuickPhrase.h"
+#include "AutoEng.h"
 
 #ifndef CODESET
 #define CODESET 14
 #endif
 
-extern char     strUserLocale[];
-
 extern Display *dpy;
 extern Window   inputWindow;
-extern Window   mainWindow;
-extern Window   aboutWindow;
 
 extern Bool     bIsUtf8;
 
@@ -63,49 +70,91 @@ extern HIDE_MAINWINDOW hideMainWindow;
 int main (int argc, char *argv[])
 {
     XEvent          event;
-    char           *imname = NULL;
-    int             i;
+    int             c; 	//ڱûĲ
     Bool            bBackground = True;
+    char	    *imname=(char *)NULL;
 
-    for (i = 1; i < argc; i++) {
-	if (!strcmp (argv[i], "-name"))
-	    imname = argv[++i];
-	else if (!strcmp (argv[i], "-nb"))
-	    bBackground = False;
-	else if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "-?")) {
-	    Usage ();
-	    return 0;
-	}
-	else if (!strcmp (argv[i], "-v")) {
-	    Version ();
-	    return 0;
-	}
+    while((c = getopt(argc, argv, "dDn:vh")) != -1) {
+        switch(c){
+            case 'd':
+                /* nothing to do */
+                break;
+            case 'D':
+                bBackground = False;
+                break;
+	    case 'n':
+	    	imname=optarg;
+		break;
+            case 'v':	//汾
+                Version();
+                return 0;
+            case 'h':	//hκβϷĲϢ
+            case '?':
+                Usage();
+                return 0;
+        }
     }
 
+    /*дڼ鵱ǰϵͳʹõǷUTFַ൱ַնִ
+     * locale charmap
+     */
     setlocale (LC_CTYPE, "");
     bIsUtf8 = (strcmp (nl_langinfo (CODESET), "UTF-8") == 0);
 
-    LoadConfig (True);
+    /* ȳʼ X ټļΪÿݼ keysym ת
+     * keycode ʱҪ Display
+     */
     if (!InitX ())
 	exit (1);
 
+    /*ûļͨǡ~/.fcitx/configļھʹӰװĿ¼п
+     * /data/config~/.fcitx/config
+     */
+    LoadConfig (True);
+
+    /*塣ʵϣǸûãʹxftȡϢ
+     * xftx11ṩĴغ
+     */
     CreateFont ();
+    //봰ڵĸ߶
     CalculateInputWindowHeight ();
+    /*ļļûõģڼ¼fctix״̬ģ
+     * ȫǻǰǵȵȡͨǡ~/.fcitx/profileļھʹӰװ
+     * Ŀ¼п/data/profile~/.fcitx/profile
+     */
     LoadProfile ();
 
+    //ֵļ
     LoadPuncDict ();
+    //س
     LoadQuickPhrase ();
+    /* ~/.fcitx/AutoEng.dat ڣ
+     *  /usr/local/share/fcitx/data/AutoEng.dat
+     * ȡҪԶתӢ״̬
+     */
+    LoadAutoEng ();
 
-    CreateMainWindow ();
-    InitGC (mainWindow);
-    CreateVKWindow ();
-    CreateInputWindow ();
-    CreateAboutWindow ();
+    //ǽĴ
 
+    CreateMainWindow ();	//ڣ뷨״̬
+    CreateVKWindow ();		//ѡʴ
+    CreateInputWindow ();	//봰
+    CreateAboutWindow ();	//ڴ
+
+    //ɫѡʴڵɫҲڡ~/.fcitx/configЩɫϢ
+    InitGC (inputWindow);
+
+    //뵽뷨飬ϵͳʹַ
     SetIM ();
-    DisplayMainWindow ();
 
-    if (!InitXIM (imname, inputWindow))
+    //ڵʾ
+    if (hideMainWindow != HM_HIDE) {
+	DisplayMainWindow ();
+	DrawMainWindow ();
+    }
+
+    //ʼ뷨
+    if (!InitXIM (inputWindow, imname))
 	exit (4);
 
     //Ժ̨ʽ
@@ -120,15 +169,17 @@ int main (int argc, char *argv[])
 	else if (id > 0)
 	    exit (0);
     }
+    
+    SetMyExceptionHandler();		//¼
 
-    SetMyExceptionHandler ();
+    //ѭXWindowϢѭ
     for (;;) {
-	XNextEvent (dpy, &event);
+	XNextEvent (dpy, &event);					//ȴһ¼
 
-	if (XFilterEvent (&event, None) == True)
+	if (XFilterEvent (&event, None) == True)	//ǳʱȴһ¼
 	    continue;
 
-	MyXEventHandler (&event);
+	MyXEventHandler (&event);					//X¼
     }
 
     return 0;
@@ -136,7 +187,12 @@ int main (int argc, char *argv[])
 
 void Usage ()
 {
-    printf ("fcitx usage:\n -name imename: \t specify the imename\n -nb :\t\t\t run as foreground \n -v:\t\t\t display the version information and exit.\n -h:\t\t\t display this help page and exit\n");
+    printf("Usage: fcitx [OPTION]\n"
+           "\t-d\t\trun as daemon(default)\n"
+           "\t-D\t\tdon't run as daemon\n"
+	   "\t-n[im name]\trun as specified name\n"
+           "\t-v\t\tdisplay the version information and exit\n"
+           "\t-h\t\tdisplay this help and exit\n");
 }
 
 void Version ()
