@@ -149,6 +149,7 @@ Bool LoadPYBaseDict (void)
     for (i = 0; i < iPYFACount; i++) {
 	fread (PYFAList[i].strMap, sizeof (char) * 2, 1, fp);
 	PYFAList[i].strMap[2] = '\0';
+	PYFAList[i].iChangeCount = 0;
 	fread (&(PYFAList[i].iBase), sizeof (int), 1, fp);
 	PYFAList[i].pyBase = (PyBase *) malloc (sizeof (PyBase) * PYFAList[i].iBase);
 	for (j = 0; j < PYFAList[i].iBase; j++) {
@@ -164,6 +165,7 @@ Bool LoadPYBaseDict (void)
 	    PYFAList[i].pyBase[j].iUserPhrase = 0;
 	    PYFAList[i].pyBase[j].userPhrase = (PyPhrase *) malloc (sizeof (PyPhrase));
 	    PYFAList[i].pyBase[j].userPhrase->next = PYFAList[i].pyBase[j].userPhrase;
+	    PYFAList[i].pyBase[j].iChangeCount = 0;
 	}
     }
 
@@ -215,6 +217,8 @@ Bool LoadPYOtherDict (void)
 #endif
 */
 	    j = GetBaseIndex (i, strBase);
+	    if (j == -1)
+		break;
 
 	    PYFAList[i].pyBase[j].iPhrase = k;
 	    PYFAList[i].pyBase[j].phrase = (PyPhrase *) malloc (sizeof (PyPhrase) * k);
@@ -239,9 +243,9 @@ Bool LoadPYOtherDict (void)
     }
 
     //濪ʼȡûʿ
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    strcat (strPath, PY_USERPHRASE_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_USERPHRASE_FILE);
     fp = fopen (strPath, "rb");
     if (fp) {
 	while (!feof (fp)) {
@@ -285,10 +289,9 @@ Bool LoadPYOtherDict (void)
     }
 
     //ȡļ
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-
-    strcat (strPath, PY_INDEX_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_INDEX_FILE);
     fp = fopen (strPath, "rb");
     if (fp) {
 	fread (&iLen, sizeof (uint), 1, fp);
@@ -316,15 +319,14 @@ Bool LoadPYOtherDict (void)
 		}
 	    }
 	}
-	
+
 	fclose (fp);
     }
 
     //ȡôʱ
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-
-    strcat (strPath, PY_FREQ_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_FREQ_FILE);
     fp = fopen (strPath, "rb");
     if (fp) {
 	pPyFreq = pyFreq;
@@ -367,9 +369,9 @@ Bool LoadPYOtherDict (void)
     }
 
     //ȡű
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    strcat (strPath, PY_SYMBOL_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_SYMBOL_FILE);
 
     if (access (strPath, 0)) {
 	strcpy (strPath, PKGDATADIR "/data/");
@@ -561,12 +563,16 @@ INPUT_RETURN_VALUE DoPYInput (int iKey)
 	    val = IRV_DISPLAY_CANDWORDS;
 	}
 	else if (iKey == RIGHT) {
+	    if (!iCodeInputCount)
+		return IRV_TO_PROCESS;
 	    if (iPYInsertPoint == strlen (strFindString))
-		return IRV_DONOT_PROCESS;
+		return IRV_DO_NOTHING;
 	    iPYInsertPoint++;
 	    val = IRV_DISPLAY_CANDWORDS;
 	}
 	else if (iKey == LEFT) {
+	    if (!iCodeInputCount)
+		return IRV_TO_PROCESS;
 	    if (iPYInsertPoint < 2) {
 		if (iPYSelected) {
 		    char            strTemp[MAX_USER_INPUT + 1];
@@ -586,7 +592,7 @@ INPUT_RETURN_VALUE DoPYInput (int iKey)
 		    val = IRV_DISPLAY_CANDWORDS;
 		}
 		else
-		    val = IRV_DONOT_PROCESS;
+		    val = IRV_DO_NOTHING;
 	    }
 	    else {
 		iPYInsertPoint--;
@@ -1062,34 +1068,36 @@ void PYCreateCandString (void)
     }
 }
 
-void PYGetCandText(int iIndex, char *strText)
+void PYGetCandText (int iIndex, char *strText)
 {
-	char           *pBase = NULL, *pPhrase;
+    char           *pBase = NULL, *pPhrase;
 
-	if (PYCandWords[iIndex].iWhich == PY_CAND_AUTO)
-		strcpy (strText, strPYAuto);
-	else {
-		pPhrase = NULL;
-		switch (PYCandWords[iIndex].iWhich) {
-			case PY_CAND_BASE:	//ϵͳ
-				pBase = PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].strHZ;
-				break;
-			case PY_CAND_USERPHRASE:	//û
-			case PY_CAND_SYMPHRASE:	//ϵͳ
-				pBase = PYFAList[PYCandWords[iIndex].cand.phrase.iPYFA].pyBase[PYCandWords[iIndex].cand.phrase.iBase].strHZ;
-				pPhrase = PYCandWords[iIndex].cand.phrase.phrase->strPhrase;
-				break;
-			case PY_CAND_FREQ:	//ǳ
-				pBase = PYCandWords[iIndex].cand.freq.hz->strHZ;
-				break;
-			case PY_CAND_SYMBOL:	//
-				pBase = PYCandWords[iIndex].cand.freq.hz->strHZ;
-				break;
-		}
-		strcpy (strText, pBase);
-		if (pPhrase)
-			strcat (strText, pPhrase);
+    if (PYCandWords[iIndex].iWhich == PY_CAND_AUTO)
+	strcpy (strText, strPYAuto);
+    else {
+	pPhrase = NULL;
+	
+	switch (PYCandWords[iIndex].iWhich) {
+	case PY_CAND_BASE:	//ϵͳ
+	    pBase = PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].strHZ;
+	    break;
+	case PY_CAND_USERPHRASE:	//û
+	case PY_CAND_SYMPHRASE:	//ϵͳ
+	    pBase = PYFAList[PYCandWords[iIndex].cand.phrase.iPYFA].pyBase[PYCandWords[iIndex].cand.phrase.iBase].strHZ;
+	    pPhrase = PYCandWords[iIndex].cand.phrase.phrase->strPhrase;
+	    break;
+	case PY_CAND_FREQ:	//ǳ
+	    pBase = PYCandWords[iIndex].cand.freq.hz->strHZ;
+	    break;
+	case PY_CAND_SYMBOL:	//
+	    pBase = PYCandWords[iIndex].cand.freq.hz->strHZ;
+	    break;
 	}
+	
+	strcpy (strText, pBase);
+	if (pPhrase)
+	    strcat (strText, pPhrase);
+    }
 }
 
 void PYSetCandWordsFlag (Bool flag)
@@ -1176,7 +1184,8 @@ void PYCreateAuto (void)
 				    }
 				    else if (strlen (phrase->strMap) <= (findMap.iHZCount - 1) * 2) {
 					if (strlen (phrase->strMap) == strlen (phraseSelected->strMap)) {
-					    if (phrase->iHit > phraseSelected->iHit) {
+					    //ȿƵƵһ
+					    if ((phrase->iHit > phraseSelected->iHit) || ((phrase->iHit == phraseSelected->iHit) && (phrase->iIndex > phraseSelected->iIndex))) {
 						baseSelected = &(PYFAList[candPos.iPYFA].pyBase[candPos.iBase]);
 						pPYFA = &PYFAList[candPos.iPYFA];
 						phraseSelected = phrase;
@@ -1212,7 +1221,10 @@ void PYCreateAuto (void)
 				    }
 				    else if (strlen (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap) <= (findMap.iHZCount - 1) * 2) {
 					if (strlen (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].strMap) == strlen (phraseSelected->strMap)) {
-					    if (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].iHit > phraseSelected->iHit) {
+					    //ȿƵƵһ
+					    if ((PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].iHit > phraseSelected->iHit) ||
+						((PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].iHit == phraseSelected->iHit) &&
+						 (PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase].iIndex > phraseSelected->iIndex))) {
 						baseSelected = &(PYFAList[candPos.iPYFA].pyBase[candPos.iBase]);
 						pPYFA = &PYFAList[candPos.iPYFA];
 						phraseSelected = &(PYFAList[candPos.iPYFA].pyBase[candPos.iBase].phrase[candPos.iPhrase]);
@@ -1417,10 +1429,8 @@ char           *PYGetCandWord (int iIndex)
 
     if (!iCandWordCount)
 	return NULL;
-
     if (iIndex > (iCandWordCount - 1))
 	iIndex = iCandWordCount - 1;
-
     switch (PYCandWords[iIndex].iWhich) {
     case PY_CAND_AUTO:
 	pBase = strPYAuto;
@@ -1432,6 +1442,7 @@ char           *PYGetCandWord (int iIndex)
 	pBaseMap = PYFAList[PYCandWords[iIndex].cand.base.iPYFA].strMap;
 	pIndex = &(PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].iIndex);
 	PYFAList[PYCandWords[iIndex].cand.base.iPYFA].pyBase[PYCandWords[iIndex].cand.base.iBase].iHit++;
+	PYFAList[PYCandWords[iIndex].cand.base.iPYFA].iChangeCount++;
 	iOrderCount++;
 	break;
     case PY_CAND_SYMPHRASE:	//ϵͳ    
@@ -1442,6 +1453,7 @@ char           *PYGetCandWord (int iIndex)
 	pPhraseMap = PYCandWords[iIndex].cand.phrase.phrase->strMap;
 	pIndex = &(PYCandWords[iIndex].cand.phrase.phrase->iIndex);
 	PYCandWords[iIndex].cand.phrase.phrase->iHit++;
+	PYFAList[PYCandWords[iIndex].cand.phrase.iPYFA].pyBase[PYCandWords[iIndex].cand.phrase.iBase].iChangeCount++;
 	iOrderCount++;
 	break;
     case PY_CAND_FREQ:		//ǳ
@@ -1459,7 +1471,6 @@ char           *PYGetCandWord (int iIndex)
 
     if (pIndex && (*pIndex != iCounter))
 	*pIndex = ++iCounter;
-
     if (iOrderCount == AUTOSAVE_ORDER_COUNT) {
 	SavePYIndex ();
 	iOrderCount = 0;
@@ -1472,21 +1483,17 @@ char           *PYGetCandWord (int iIndex)
     strcpy (messageDown[uMessageDown].strMsg, pBase);
     if (pPhrase)
 	strcat (messageDown[uMessageDown].strMsg, pPhrase);
-
     strcpy (strHZString, pBase);
     if (pPhrase)
 	strcat (strHZString, pPhrase);
-
     iLen = strlen (strHZString) / 2;
     if (iLen == findMap.iHZCount || PYCandWords[iIndex].iWhich == PY_CAND_SYMBOL) {
 	strPYAuto[0] = '\0';
 	for (iLen = 0; iLen < iPYSelected; iLen++)
 	    strcat (strPYAuto, pySelected[iLen].strHZ);
 	strcat (strPYAuto, strHZString);
-
 	ParsePY (strCodeInput, &findMap, PY_PARSE_INPUT_USER);
 	strHZString[0] = '\0';
-
 	for (i = 0; i < iPYSelected; i++)
 	    strcat (strHZString, pySelected[i].strMap);
 	if (pBaseMap)
@@ -1496,16 +1503,12 @@ char           *PYGetCandWord (int iIndex)
 	//if (!bSingleHZMode && bAddNewPhrase && (strlen (strPYAuto) <= (MAX_PY_PHRASE_LENGTH * 2)))
 	if (bAddNewPhrase && (strlen (strPYAuto) <= (MAX_PY_PHRASE_LENGTH * 2)))
 	    PYAddUserPhrase (strPYAuto, strHZString);
-
 	uMessageDown = 0;
 	uMessageUp = 0;
-
 	if (bUseLegend) {
 	    strcpy (strPYLegendSource, strPYAuto);
 	    strcpy (strPYLegendMap, strHZString);
-
 	    PYGetLegendCandWords (SM_FIRST);
-
 	    iPYInsertPoint = 0;
 	    strFindString[0] = '\0';
 	}
@@ -1518,35 +1521,29 @@ char           *PYGetCandWord (int iIndex)
     pySelected[iPYSelected].strMap[0] = '\0';
     for (i = 0; i < iLen; i++)
 	strcat (pySelected[iPYSelected].strPY, findMap.strPYParsed[i]);
-
     if (pBaseMap)
 	strcat (pySelected[iPYSelected].strMap, pBaseMap);
     if (pPhraseMap)
 	strcat (pySelected[iPYSelected].strMap, pPhraseMap);
     strcpy (pySelected[iPYSelected].strHZ, strHZString);
     iPYSelected++;
-
     strFindString[0] = '\0';
     for (; i < findMap.iHZCount; i++)
 	strcat (strFindString, findMap.strPYParsed[i]);
-
     DoPYInput (-1);
     iPYInsertPoint = strlen (strFindString);
-
     return NULL;
 }
 
 void PYGetCandWordsForward (void)
 {
-    //if (!bSingleHZMode) {
-	if (pCurFreq && pCurFreq->bIsSym)
-	    PYGetSymCandWords (SM_NEXT);
-	else {
-	    PYGetPhraseCandWords (SM_NEXT);
-	    if (pCurFreq)
-		PYGetFreqCandWords (SM_NEXT);
-	}
-    //}
+    if (pCurFreq && pCurFreq->bIsSym)
+	PYGetSymCandWords (SM_NEXT);
+    else {
+	PYGetPhraseCandWords (SM_NEXT);
+	if (pCurFreq)
+	    PYGetFreqCandWords (SM_NEXT);
+    }
 
     if (!(pCurFreq && pCurFreq->bIsSym))
 	PYGetBaseCandWords (SM_NEXT);
@@ -1559,7 +1556,7 @@ void PYGetCandWordsBackward (void)
 	PYGetSymCandWords (SM_PREV);
     else {
 	//if (!bSingleHZMode)
-	    PYGetFreqCandWords (SM_PREV);
+	PYGetFreqCandWords (SM_PREV);
 	PYGetBaseCandWords (SM_PREV);
 	//if (iCandWordCount == iMaxCandWord || bSingleHZMode)
 	if (iCandWordCount == iMaxCandWord)
@@ -1582,7 +1579,6 @@ Bool PYCheckNextCandPage (void)
     str[1] = findMap.strMap[0][1];
     str[2] = '\0';
     strMap[0] = '\0';
-
     if (pCurFreq && pCurFreq->bIsSym) {
 	hz = pCurFreq->HZList->next;
 	for (val = 0; val < pCurFreq->iCount; val++) {
@@ -1664,15 +1660,12 @@ void PYGetPhraseCandWords (SEARCH_MODE mode)
 
     if (findMap.iHZCount == 1)
 	return;
-
     str[0] = findMap.strMap[0][0];
     str[1] = findMap.strMap[0][1];
     str[2] = '\0';
     strMap[0] = '\0';
-
     for (val = 1; val < findMap.iHZCount; val++)
 	strcat (strMap, findMap.strMap[val]);
-
     for (candPos.iPYFA = 0; candPos.iPYFA < iPYFACount; candPos.iPYFA++) {
 	if (!Cmp2Map (PYFAList[candPos.iPYFA].strMap, str)) {
 	    for (candPos.iBase = 0; candPos.iBase < PYFAList[candPos.iPYFA].iBase; candPos.iBase++) {
@@ -1775,7 +1768,6 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 		return False;
 	}
 	break;
-
 	//ֿԷһ
     case AD_FAST:
 	if (mode == SM_PREV) {
@@ -1811,7 +1803,6 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
 	    }
 	    else if (iCandWordCount == iMaxCandWord)
 		i--;
-
 	}
 	else {
 	    for (i = 0; i < iCandWordCount; i++) {
@@ -1964,10 +1955,8 @@ Bool PYAddPhraseCandWord (PYCandIndex pos, PyPhrase * phrase, SEARCH_MODE mode,
     PYCandWords[i].cand.phrase.phrase = phrase;
     PYCandWords[i].cand.phrase.iPYFA = pos.iPYFA;
     PYCandWords[i].cand.phrase.iBase = pos.iBase;
-
     if (iCandWordCount != iMaxCandWord)
 	iCandWordCount++;
-
     return True;
 }
 
@@ -2011,19 +2000,15 @@ Bool PYAddSymCandWord (HZ * hz, SEARCH_MODE mode)
     else {
 	if (iCandWordCount == iMaxCandWord)
 	    return False;
-
 	i = iCandWordCount;
-
 	for (j = iCandWordCount - 1; j > i; j--)
 	    PYCandWords[j].cand.sym.hz = PYCandWords[j - 1].cand.sym.hz;
     }
 
     PYCandWords[i].iWhich = PY_CAND_SYMBOL;
     PYCandWords[i].cand.sym.hz = hz;
-
     if (iCandWordCount != iMaxCandWord)
 	iCandWordCount++;
-
     return True;
 }
 
@@ -2031,7 +2016,9 @@ Bool PYAddSymCandWord (HZ * hz, SEARCH_MODE mode)
 
 void PYGetBaseCandWords (SEARCH_MODE mode)
 {
-    PYCandIndex     candPos = { 0, 0, 0 };
+    PYCandIndex     candPos = {
+	0, 0, 0
+    };
     char            str[3];
 
     str[0] = findMap.strMap[0][0];
@@ -2244,10 +2231,8 @@ Bool PYAddBaseCandWord (PYCandIndex pos, SEARCH_MODE mode)
     PYCandWords[i].iWhich = PY_CAND_BASE;
     PYCandWords[i].cand.base.iPYFA = pos.iPYFA;
     PYCandWords[i].cand.base.iBase = pos.iBase;
-
     if (iCandWordCount != iMaxCandWord)
 	iCandWordCount++;
-
     return True;
 }
 
@@ -2426,10 +2411,8 @@ Bool PYAddFreqCandWord (HZ * hz, char *strPY, SEARCH_MODE mode)
     PYCandWords[i].iWhich = PY_CAND_FREQ;
     PYCandWords[i].cand.freq.hz = hz;
     PYCandWords[i].cand.freq.strPY = strPY;
-
     if (iCandWordCount != iMaxCandWord)
 	iCandWordCount++;
-
     return True;
 }
 
@@ -2446,7 +2429,6 @@ Bool PYAddUserPhrase (char *phrase, char *map)
     //֣ɴ
     if (strlen (phrase) < 4)
 	return False;
-
     str[0] = map[0];
     str[1] = map[1];
     str[2] = '\0';
@@ -2454,7 +2436,6 @@ Bool PYAddUserPhrase (char *phrase, char *map)
     str[0] = phrase[0];
     str[1] = phrase[1];
     j = GetBaseIndex (i, str);;
-
     //жϸôǷѾڿ
     //ȣǲû
     userPhrase = PYFAList[i].pyBase[j].userPhrase->next;
@@ -2468,7 +2449,6 @@ Bool PYAddUserPhrase (char *phrase, char *map)
     for (k = 0; k < PYFAList[i].pyBase[j].iPhrase; k++)
 	if (!strcmp (map + 2, PYFAList[i].pyBase[j].phrase[k].strMap) && !strcmp (phrase + 2, PYFAList[i].pyBase[j].phrase[k].strPhrase))
 	    return False;
-
     //潫ӵб
     newPhrase = (PyPhrase *) malloc (sizeof (PyPhrase));
     newPhrase->strMap = (char *) malloc (sizeof (char) * strlen (map + 2) + 1);
@@ -2512,13 +2492,10 @@ void PYDelUserPhrase (int iPYFA, int iBase, PyPhrase * phrase)
     }
     if (!temp)
 	return;
-
     temp->next = phrase->next;
-
     free (phrase->strPhrase);
     free (phrase->strMap);
     free (phrase);
-
     PYFAList[iPYFA].pyBase[iBase].iUserPhrase--;
     iNewPYPhraseCount++;
     if (iNewPYPhraseCount == AUTOSAVE_PHRASE_COUNT) {
@@ -2585,10 +2562,9 @@ void SavePYUserPhrase (void)
     }
 
     fclose (fp);
-
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    strcat (strPath, PY_USERPHRASE_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_USERPHRASE_FILE);
     if (access (strPath, 0))
 	unlink (strPath);
     rename (strPathTemp, strPath);
@@ -2599,23 +2575,23 @@ void SavePYFreq (void)
     int             i, j, k;
     char            strPath[PATH_MAX];
     char            strPathTemp[PATH_MAX];
+    int             iStrLen = PATH_MAX;
     FILE           *fp;
     PyFreq         *pPyFreq;
     HZ             *hz;
 
-    strcpy (strPathTemp, (char *) getenv ("HOME"));
-    strcat (strPathTemp, "/.fcitx/");
+    snprintf(strPathTemp, iStrLen, "%s/.fcitx/",
+             getenv ("HOME"));
+    iStrLen -= strlen(strPathTemp);
     if (access (strPathTemp, 0))
 	mkdir (strPathTemp, S_IRWXU);
-    strcat (strPathTemp, TEMP_FILE);
-
+    strncat(strPathTemp, TEMP_FILE, iStrLen);
     fp = fopen (strPathTemp, "wb");
     if (!fp) {
 	fprintf (stderr, "޷泣ôʱ%s\n", strPathTemp);
 	return;
     }
     i = 0;
-
     pPyFreq = pyFreq->next;
     while (pPyFreq) {
 	if (!pPyFreq->bIsSym)
@@ -2623,14 +2599,12 @@ void SavePYFreq (void)
 	pPyFreq = pPyFreq->next;
     }
     fwrite (&i, sizeof (uint), 1, fp);
-
     pPyFreq = pyFreq->next;
     while (pPyFreq) {
 	if (!pPyFreq->bIsSym) {
 	    fwrite (pPyFreq->strPY, sizeof (char) * 11, 1, fp);
 	    j = pPyFreq->iCount;
 	    fwrite (&j, sizeof (int), 1, fp);
-
 	    hz = pPyFreq->HZList->next;
 	    for (k = 0; k < pPyFreq->iCount; k++) {
 		fwrite (hz->strHZ, sizeof (char) * 2, 1, fp);
@@ -2640,7 +2614,6 @@ void SavePYFreq (void)
 		fwrite (&j, sizeof (int), 1, fp);
 		j = hz->iIndex;
 		fwrite (&j, sizeof (int), 1, fp);
-
 		hz = hz->next;
 	    }
 	}
@@ -2648,10 +2621,9 @@ void SavePYFreq (void)
     }
 
     fclose (fp);
-
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    strcat (strPath, PY_FREQ_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_FREQ_FILE);
     if (access (strPath, 0))
 	unlink (strPath);
     rename (strPathTemp, strPath);
@@ -2662,16 +2634,18 @@ void SavePYFreq (void)
  */
 void SavePYIndex (void)
 {
-    int             i, j, k, l, m;
+    int             i, j, k, l;
     char            strPath[PATH_MAX];
     char            strPathTemp[PATH_MAX];
+    int             iStrLen = PATH_MAX;
     FILE           *fp;
 
-    strcpy (strPathTemp, (char *) getenv ("HOME"));
-    strcat (strPathTemp, "/.fcitx/");
+    snprintf(strPathTemp, iStrLen, "%s/.fcitx/",
+             getenv ("HOME"));
+    iStrLen -= strlen(strPathTemp);
     if (access (strPathTemp, 0))
 	mkdir (strPathTemp, S_IRWXU);
-    strcat (strPathTemp, TEMP_FILE);
+    strncat(strPathTemp, TEMP_FILE, iStrLen);
     fp = fopen (strPathTemp, "wb");
     if (!fp) {
 	fprintf (stderr, "޷ļ%s\n", strPathTemp);
@@ -2680,45 +2654,49 @@ void SavePYIndex (void)
 
     //
     fwrite (&iCounter, sizeof (uint), 1, fp);
-
     //ȱΪ0ĵ
     k = -1;
     for (i = 0; i < iPYFACount; i++) {
-	for (j = 0; j < PYFAList[i].iBase; j++) {
-	    l = PYFAList[i].pyBase[j].iIndex;
-	    m = PYFAList[i].pyBase[j].iHit;
-	    if (l || m) {
-		fwrite (&i, sizeof (int), 1, fp);
-		fwrite (&j, sizeof (int), 1, fp);
-		fwrite (&k, sizeof (int), 1, fp);
-		fwrite (&l, sizeof (uint), 1, fp);
-		fwrite (&m, sizeof (uint), 1, fp);
+	if (PYFAList[i].iChangeCount) {
+	    for (j = 0; j < PYFAList[i].iBase; j++) {
+		if (PYFAList[i].pyBase[j].iIndex) {
+		    fwrite (&i, sizeof (int), 1, fp);
+		    fwrite (&j, sizeof (int), 1, fp);
+		    fwrite (&k, sizeof (int), 1, fp);
+		    l = PYFAList[i].pyBase[j].iIndex;
+		    fwrite (&l, sizeof (uint), 1, fp);
+		    l = PYFAList[i].pyBase[j].iHit;
+		    fwrite (&l, sizeof (uint), 1, fp);
+		}
 	    }
 	}
     }
 
     //ٱΪ0ϵͳ
     for (i = 0; i < iPYFACount; i++) {
-	for (j = 0; j < PYFAList[i].iBase; j++) {
-	    for (k = 0; k < PYFAList[i].pyBase[j].iPhrase; k++) {
-		l = PYFAList[i].pyBase[j].phrase[k].iIndex;
-		m = PYFAList[i].pyBase[j].phrase[k].iHit;
-		if (l || m) {
-		    fwrite (&i, sizeof (int), 1, fp);
-		    fwrite (&j, sizeof (int), 1, fp);
-		    fwrite (&k, sizeof (int), 1, fp);
-		    fwrite (&l, sizeof (uint), 1, fp);
-		    fwrite (&m, sizeof (uint), 1, fp);
+	if (PYFAList[i].iChangeCount) {
+	    for (j = 0; j < PYFAList[i].iBase; j++) {
+		if (PYFAList[i].pyBase[j].iChangeCount) {
+		    for (k = 0; k < PYFAList[i].pyBase[j].iPhrase; k++) {
+			if (PYFAList[i].pyBase[j].phrase[k].iIndex) {
+			    fwrite (&i, sizeof (int), 1, fp);
+			    fwrite (&j, sizeof (int), 1, fp);
+			    fwrite (&k, sizeof (int), 1, fp);
+			    l = PYFAList[i].pyBase[j].phrase[k].iIndex;
+			    fwrite (&l, sizeof (uint), 1, fp);
+			    l = PYFAList[i].pyBase[j].phrase[k].iHit;
+			    fwrite (&l, sizeof (uint), 1, fp);
+			}
+		    }
 		}
 	    }
 	}
     }
 
     fclose (fp);
-
-    strcpy (strPath, (char *) getenv ("HOME"));
-    strcat (strPath, "/.fcitx/");
-    strcat (strPath, PY_INDEX_FILE);
+    snprintf(strPath, sizeof(strPath), "%s/.fcitx/%s",
+             getenv ("HOME"),
+             PY_INDEX_FILE);
     if (access (strPath, 0))
 	unlink (strPath);
     rename (strPathTemp, strPath);
@@ -2758,9 +2736,7 @@ void PYAddFreq (int iIndex)
     //iָʾǷҪµĳ
     if (i < 0)
 	return;
-
     PYSetCandWordsFlag (False);
-
     //ҪӸ֣ʱֱȻϵͳ
     if (!pCurFreq) {
 	freq = (PyFreq *) malloc (sizeof (PyFreq));
@@ -2770,14 +2746,11 @@ void PYAddFreq (int iIndex)
 	freq->next = NULL;
 	freq->iCount = 0;
 	freq->bIsSym = False;
-
 	pCurFreq = pyFreq;
 	for (i = 0; i < iPYFreqCount; i++)
 	    pCurFreq = pCurFreq->next;
 	pCurFreq->next = freq;
-
 	iPYFreqCount++;
-
 	pCurFreq = freq;
     }
 
@@ -2788,16 +2761,13 @@ void PYAddFreq (int iIndex)
     HZTemp->iIndex = 0;
     HZTemp->flag = 0;
     HZTemp->next = NULL;
-
     //HZTempӵβ
     hz = pCurFreq->HZList;
     for (i = 0; i < pCurFreq->iCount; i++)
 	hz = hz->next;
-
     hz->next = HZTemp;
     pCurFreq->iCount++;
     iNewFreqCount++;
-
     if (iNewFreqCount == AUTOSAVE_FREQ_COUNT) {
 	SavePYFreq ();
 	iNewFreqCount = 0;
@@ -2815,20 +2785,15 @@ void PYDelFreq (int iIndex)
     //ȣǲѾڳֱ
     if (PYCandWords[iIndex].iWhich != PY_CAND_FREQ)
 	return;
-
     PYSetCandWordsFlag (False);
-
     //ҵҪɾֵλ
     hz = pCurFreq->HZList;
     while (hz->next != PYCandWords[iIndex].cand.freq.hz)
 	hz = hz->next;
-
     hz->next = PYCandWords[iIndex].cand.freq.hz->next;
     free (PYCandWords[iIndex].cand.freq.hz);
     pCurFreq->iCount--;
-
     iNewFreqCount++;
-
     if (iNewFreqCount == AUTOSAVE_FREQ_COUNT) {
 	SavePYFreq ();
 	iNewFreqCount = 0;
@@ -2845,7 +2810,6 @@ Bool PYIsInFreq (char *strHZ)
 
     if (!pCurFreq || pCurFreq->bIsSym)
 	return False;
-
     hz = pCurFreq->HZList->next;
     for (i = 0; i < pCurFreq->iCount; i++) {
 	if (!strcmp (strHZ, hz->strHZ))
@@ -2868,14 +2832,11 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
 
     if (!strPYLegendSource[0])
 	return IRV_TO_PROCESS;
-
     if (mode == SM_FIRST) {
 	iLegendCandPageCount = 0;
 	iLegendCandWordCount = 0;
 	iCurrentLegendCandPage = 0;
-
 	PYResetFlags ();
-
 	pyBaseForLengend = NULL;
 	for (i = 0; i < iPYFACount; i++) {
 	    if (!strncmp (strPYLegendMap, PYFAList[i].strMap, 2)) {
@@ -2891,24 +2852,20 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
       _HIT:
 	if (!pyBaseForLengend)
 	    return IRV_TO_PROCESS;
-
 	bShowCursor = False;
     }
     else {
 	if (!iLegendCandPageCount)
 	    return IRV_TO_PROCESS;
-
 	if (mode == SM_NEXT) {
 	    if (iCurrentLegendCandPage == iLegendCandPageCount)
 		return IRV_DO_NOTHING;
-
 	    iLegendCandWordCount = 0;
 	    iCurrentLegendCandPage++;
 	}
 	else {
 	    if (!iCurrentLegendCandPage)
 		return IRV_DO_NOTHING;
-
 	    iCurrentLegendCandPage--;
 	    PYSetLegendCandWordsFlag (False);
 	}
@@ -2948,7 +2905,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
     }
 
     PYSetLegendCandWordsFlag (True);
-
     if (!bDisablePagingInLegend && mode != SM_PREV && iCurrentLegendCandPage == iLegendCandPageCount) {
 	for (i = 0; i < pyBaseForLengend->iPhrase; i++) {
 	    if (strlen (strPYLegendSource) == 2) {
@@ -2990,7 +2946,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
     messageUp[0].type = MSG_TIPS;
     strcpy (messageUp[1].strMsg, strPYLegendSource);
     messageUp[1].type = MSG_INPUT;
-
     strTemp[1] = '\0';
     uMessageDown = 0;
     for (i = 0; i < iLegendCandWordCount; i++) {
@@ -2999,7 +2954,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
 	    strTemp[0] = '0';
 	strcpy (messageDown[uMessageDown].strMsg, strTemp);
 	messageDown[uMessageDown++].type = MSG_INDEX;
-
 	strcpy (messageDown[uMessageDown].strMsg, PYLegendCandWords[i].phrase->strPhrase + PYLegendCandWords[i].iLength);
 	if (i != (iLegendCandWordCount - 1)) {
 	    strcat (messageDown[uMessageDown].strMsg, " ");
@@ -3008,7 +2962,6 @@ INPUT_RETURN_VALUE PYGetLegendCandWords (SEARCH_MODE mode)
     }
 
     bIsInLegend = (iLegendCandWordCount != 0);
-
     return IRV_DISPLAY_CANDWORDS;
 }
 
@@ -3067,10 +3020,8 @@ Bool PYAddLengendCandWord (PyPhrase * phrase, SEARCH_MODE mode)
 
     PYLegendCandWords[i].phrase = phrase;
     PYLegendCandWords[i].iLength = strlen (strPYLegendSource) - 2;
-
     if (iLegendCandWordCount != iMaxCandWord)
 	iLegendCandWordCount++;
-
     return True;
 }
 
@@ -3082,7 +3033,6 @@ char           *PYGetLegendCandWord (int iIndex)
 	strcpy (strPYLegendSource, PYLegendCandWords[iIndex].phrase->strPhrase + PYLegendCandWords[iIndex].iLength);
 	strcpy (strPYLegendMap, PYLegendCandWords[iIndex].phrase->strMap + PYLegendCandWords[iIndex].iLength);
 	PYGetLegendCandWords (SM_FIRST);
-
 	return strPYLegendSource;
     }
 
