@@ -27,7 +27,7 @@
 
 #define TABLE_CONFIG_FILENAME "tables.conf"
 
-#define MAX_CODE_LENGTH  12
+#define MAX_CODE_LENGTH  30
 #define PHRASE_MAX_LENGTH 10
 #define FH_MAX_LENGTH  10
 #define TABLE_AUTO_SAVE_AFTER 1024
@@ -52,10 +52,12 @@ typedef struct _TABLE {
     char            strName[MAX_IM_NAME + 1];
     char           *strInputCode;
     unsigned char   iCodeLength;
+    unsigned char   iPYCodeLength;
+    char           *strEndCode;	//ֹ¸ü൱üٰһո
     char           *strIgnoreChars;
     char            cMatchingKey;
     char            strSymbol[MAX_CODE_LENGTH + 1];
-    char            cPinyin;
+    char            cPinyin;	//ü󣬱ʾʱƴ״̬
     unsigned char   bRule;
 
     RULE           *rule;	//ʹ
@@ -64,7 +66,8 @@ typedef struct _TABLE {
     ADJUSTORDER     tableOrder;
 
     Bool            bUsePY;	//ʹƴ
-    Bool            bTableAutoSendToClient;	//Զ
+    INT8            iTableAutoSendToClient;	//Զ
+    INT8            iTableAutoSendToClientWhenNone;	//Զ
     Bool            bUseMatchingKey;	//Ƿģƥ
     Bool            bAutoPhrase;	//ǷԶ
     INT8            iSaveAutoPhraseAfter;	//ѡNκ󱣴Զ飬0-棬1-
@@ -72,6 +75,9 @@ typedef struct _TABLE {
     INT8            iAutoPhrase;	//Զʳ
     Bool            bTableExactMatch;	//Ƿֻʾȷƥĺѡ/
     Bool            bPromptTableCode;	//ϺǷʾ
+
+    Bool            bHasPinyin;		//ǸǷƴ
+    char            choose[11];		//ѡ
 } TABLE;
 
 typedef struct _RECORD {
@@ -82,6 +88,7 @@ typedef struct _RECORD {
     unsigned int    iHit;
     unsigned int    iIndex;
     unsigned int    flag:1;
+    unsigned int    bPinyin:1;
 } RECORD;
 
 /* ݼһ򵥵ָüʼĵһ¼ */
@@ -116,7 +123,7 @@ typedef struct _TABLECANDWORD {
 typedef enum {
     CT_NORMAL = 0,
     CT_AUTOPHRASE,
-    CT_PYPHRASE
+    CT_PYPHRASE			//ʱƴתĺѡ/
 } CANDTYPE;
 
 void            LoadTableInfo (void);
@@ -126,6 +133,7 @@ void            FreeTableIM (void);
 void            SaveTableDict (void);
 Bool            IsInputKey (int iKey);
 Bool            IsIgnoreChar (char cChar);
+Bool            IsEndKey (char cChar);
 INPUT_RETURN_VALUE DoTableInput (int iKey);
 INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode);
 void            TableAddCandWord (RECORD * wbRecord, SEARCH_MODE mode);
@@ -147,7 +155,9 @@ void            TableDelPhrase (RECORD * record);
 RECORD         *TableHasPhrase (char *strCode, char *strHZ);
 RECORD         *TableFindPhrase (char *strHZ);
 void            TableInsertPhrase (char *strCode, char *strHZ);
+char	       *_TableGetCandWord (int iIndex, Bool _bLegend);		//Internal
 char           *TableGetCandWord (int iIndex);
+void		TableUpdateHitFrequency (RECORD * record);
 void            TableCreateNewPhrase (void);
 void            TableCreatePhraseCode (char *strHZ);
 Bool            TablePhraseTips (void);
