@@ -17,10 +17,21 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+/**
+ * @file   tools.h
+ * @author Yuking yuking_net@sohu.com
+ * @date   2008-1-16
+ * 
+ * @brief  ļд
+ * 
+ * 
+ */
+
 #ifndef _TOOLS_H
 #define _TOOLS_H
 
 #include <stdio.h>
+#include <errno.h>
 #include "ime.h"
 
 #define TABLE_GBKS2T "gbks2t.tab"
