@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
@@ -6,7 +7,7 @@ int main(int argc, char *argv[])
 {
     FILE *fp;
     char strCode[10];
-    char strHZ[50];
+    char strHZ[256];
     int s;
     
     if ( argc!=2 ) {
@@ -26,8 +27,10 @@ int main(int argc, char *argv[])
 	    if ( isalpha(strHZ[0]) )
 		    strcpy(strCode, strHZ);
 	    else {
-		    printf("%s %s\n", strCode,strHZ);
-		    s++;
+		if(strlen(strCode)>=10 || strlen(strHZ)>=50)
+		    exit(0);
+		printf("%s %s\n", strCode,strHZ);
+		s++;
 	    }
     }
     
